/*
 * da9030.S
 *
 * Configuration settings for the CompuLab CM-X300 device
 *
 * (C) Copyright 2007 CompuLab, by Igor Vaisbein
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <config.h>
#include <version.h>		
#include <asm/arch/pxa-regs.h>
#include <da9030.h>

.globl da9030_set_reg
.globl da9030_get_reg

getbytei2c:
		ldr	r3, =PWRICR
		ldr	r0, [r3]
		orr	r0, r0, r1
		bic	r0, r0, r2
		str	r0, [r3]
		orr	r0, r0, #ICR_TB
		str	r0, [r3]
		mov	r2, #0x100000

waitforcvfulli2c:
		ldr	r3, =PWRISR
		ldr	r1, [r3]
		/* take it from the top if we don't get empty after a while */
		subs	r2, r2, #1
		beq	errorrx
		tst	r1, #ISR_IRF
		beq	waitforcvfulli2c
		ldr	r2, =PWRIDBR
		ldr	r0, [r2]	
		orr	r1, r1, #ISR_IRF
		str	r1, [r3]
		mov	pc, lr
errorrx:
		add	r8, r8, #1		/* set Error condition */
		str	r1, [r3]

		ldr	r2, =PWRIDBR		/* Load Data but ignore */
		ldr	r0, [r2]

		mov	r0, #0			/* return 0 */
		mov	pc, lr


sendbytei2c:
		ldr	r3, =PWRIDBR
		str	r0, [r3]
		ldr	r3, =PWRICR
		ldr	r0, [r3]
		orr	r0, r0, r1
		bic	r0, r0, r2
		str	r0, [r3]
		orr	r0, r0, #ICR_TB
		str	r0, [r3]
		mov	r2, #0x100000

waitfortxemptyi2c:
		ldr	r0, =PWRISR
		ldr	r1, [r0]
		/* take it from the top if we don't get empty after a while */
		subs	r2, r2, #1
		beq	errortx
		tst	r1, #ISR_ITE
		beq	waitfortxemptyi2c
		orr	r1, r1, #ISR_ITE
		str	r1, [r0]
		mov	pc, lr
errortx:
		add	r8, r8, #1		/* set Error condition */
		str	r1, [r0]

		mov	pc, lr

finishi2c:
		ldr	r3, =PWRICR
		ldr	r2, [r3]
		bic	r2, r2, #(ICR_START | ICR_STOP | ICR_ACKNAK)
		str	r2, [r3]
		mov	pc, lr


/* Set DA9030 register function */
/* The register address should be in r0 */
/* The register value should be in r1 */
da9030_set_reg:

		mov	r5, r0
		mov	r6, r1
		mov	r8, #0			/* No Error */

		ldr	r0, =DA9030_ADDR
		mov	r1, #ICR_START
		ldr	r2, =(ICR_STOP | ICR_ALDIE | ICR_ACKNAK)

		mov	r7, lr
		bl	sendbytei2c
		mov	lr, r7

		cmp	r8, #1			/* Test for Error */
		beq	exit_set_reg		/* goto Exit */

		mov	r0, r5
		mov	r1, #0
		mov	r2, #(ICR_STOP | ICR_START)

		mov	r7, lr
		bl	sendbytei2c
		mov	lr, r7

		cmp	r8, #1			/* Test for Error */
		beq	exit_set_reg		/* goto Exit */

		mov	r0, r6
		and	r0, r0, #0xff
		mov	r1, #0
		mov	r2, #(ICR_START | ICR_STOP)

		mov	r7, lr
		bl	sendbytei2c
		mov	lr, r7

exit_set_reg:

		mov 	r7, lr
		bl	finishi2c
		mov 	lr, r7

  		mov 	pc, lr

/* Get DA9030 register function */
/* The register address should be in r0 */
/* The register value is returned in r0 */
da9030_get_reg:

		mov	r5, r0
		mov	r8, #0			/* No Error */

		ldr	r0, =DA9030_ADDR
		mov	r1, #ICR_START
		ldr	r2, =(ICR_STOP | ICR_ALDIE | ICR_ACKNAK)

		mov	r7, lr
		bl	sendbytei2c
		mov	lr, r7

		cmp	r8, #1			/* Test for Error */
		beq	exit_get_reg		/* goto Exit */

		mov	r0, r5
		mov	r1, #0
		mov	r2, #(ICR_STOP | ICR_START)

		mov	r7, lr
		bl	sendbytei2c
		mov	lr, r7

		cmp	r8, #1			/* Test for Error */
		beq	exit_get_reg		/* goto Exit */

		ldr	r0, =DA9030_ADDR
		orr	r0, r0, #0x01
		mov	r1, #ICR_START
		ldr	r2, =(ICR_STOP | ICR_ALDIE | ICR_ACKNAK)

		mov	r7, lr
		bl	sendbytei2c
		mov	lr, r7

		cmp	r8, #1			/* Test for Error */
		beq	exit_get_reg		/* goto Exit */

		ldr	r1, =(ICR_ALDIE | ICR_ACKNAK)
		mov	r2, #(ICR_START | ICR_STOP)

		mov	r7, lr
		bl	getbytei2c
		mov	lr, r7

		and	r0, r0, #0xff

exit_get_reg:
		bl	finishi2c
		mov	lr, r7

		mov	pc, lr

