/*
 * board/cmx300/gpio.c
 *
 * User custom GPIO configuration for the CompuLab CM-X300 module
 *
 * (C) Copyright 2010
 * Yulia Vilensky, CompuLab, vilensky@compulab.co.il
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */
#include <common.h>
#include "cmx300_common.h"
#include "gpio.h"
#include "mfp.h"

#define GPIO_OPTIONS_ENV_VAR_NAME "customgpio"

#define GPIO_NUM 128

/*
 * The first bit is set - the GPIO is available to the user disregarding
 * the options. If no bits are set - the GPIO isn't available.
 */
#define GPIO_OPT_MASK_AVAILABLE 0x80
/* GPIO is available to the user only with PXA300 */
#define GPIO_OPT_MASK_PXA300 0x40
/* GPIO is available to the user only with PXA310 */
#define GPIO_OPT_MASK_PXA310 0x20
/* GPIO is available to the user only without AT option in the serial number */
#define GPIO_OPT_MASK_AT 0x10
/* GPIO is available to the user only without W option in the serial number */
#define GPIO_OPT_MASK_W 0x08

#define INVAL 0x00
#define AVAIL GPIO_OPT_MASK_AVAILABLE
#define PXA30 GPIO_OPT_MASK_PXA300
#define PXA31 GPIO_OPT_MASK_PXA310
#define NO_AT GPIO_OPT_MASK_AT
#define NO__W GPIO_OPT_MASK_W

typedef struct serial_opt {
	int at;
	int w;
} serial_opt_t;

static unsigned char gpio_opt_rev120[] = {
/*	0      1      2      3      4      5      6      7      8      9      */
/* 0 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL,
/* 1 */ INVAL, INVAL, INVAL, INVAL, INVAL, AVAIL, INVAL, INVAL, INVAL, AVAIL,
/* 2 */ INVAL, AVAIL, AVAIL, NO_AT, INVAL, NO_AT, INVAL, NO_AT, NO_AT, NO_AT,
/* 3 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL,
/* 4 */ INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, AVAIL,

/* 5 */ INVAL, INVAL, INVAL, INVAL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL,
/* 6 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL,
/* 7 */ INVAL, INVAL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL, INVAL, INVAL, INVAL,
/* 8 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL,
/* 9 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL, INVAL, INVAL, INVAL, INVAL,

/*10 */ INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, AVAIL, INVAL, INVAL, INVAL,
/*11 */ INVAL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL,
/*12 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL,
};

static unsigned char gpio_opt_rev130[] = {
/*	0      1      2      3      4      5      6      7      8      9      */
/* 0 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL,
/* 1 */ INVAL, INVAL, INVAL, INVAL, INVAL, AVAIL, INVAL, INVAL, INVAL, AVAIL,
/* 2 */ INVAL, AVAIL, AVAIL, NO_AT, INVAL, NO_AT, INVAL, NO_AT, NO_AT, NO_AT,
/* 3 */ PXA30, PXA30, PXA30, PXA30, PXA30, PXA30, PXA30, PXA30, INVAL, PXA31,
/* 4 */ PXA31, PXA31, PXA31, PXA31, PXA31, PXA31, PXA31, PXA31, PXA31, AVAIL,

/* 5 */ PXA31, PXA31, PXA31, INVAL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL,
/* 6 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL,
/* 7 */ INVAL, INVAL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL, INVAL, INVAL, INVAL,
/* 8 */ PXA30, PXA30, PXA30, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL,
/* 9 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL, INVAL, INVAL, INVAL, INVAL,

/*10 */ INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, INVAL, INVAL,
/*11 */ INVAL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL,
/*12 */ AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, AVAIL, INVAL, INVAL,
};

extern u32 get_board_rev(void);
extern int get_ser_option(const char o, const char *s);

static int is_valid_gpio(serial_opt_t *ser_num,
			  int gpio,
			  unsigned char *options)
{
	if (gpio > 127)
		return 0;

	if (options[gpio] == 0)
		return 0;

	if (options[gpio] & GPIO_OPT_MASK_AVAILABLE)
		return 1;

	if (cpu_is_pxa310() && (options[gpio] & GPIO_OPT_MASK_PXA300))
		return 0;

	if (cpu_is_pxa300() && (options[gpio] & GPIO_OPT_MASK_PXA310))
		return 0;

	if ((options[gpio] & GPIO_OPT_MASK_AT) && ser_num->at)
		return 0;

	if ((options[gpio] & GPIO_OPT_MASK_W) && ser_num->w)
		return 0;

	return 1;
}

static char gpio_opt_str[256];

static void set_gpio(char		*curr_gpio_opt,
		      serial_opt_t	*serialnr,
		      unsigned char	*options)
{
	char *gpio_str;
	char *direction;
	char *value_str;
	int gpio, value;

	gpio_str = strsep(&curr_gpio_opt, " ");
	if (gpio_str == NULL)
		return;

	/* Check if the user is eligible to set this GPIO */
	gpio = simple_strtoul(gpio_str, NULL, 10);
	if (!is_valid_gpio(serialnr, gpio, options))
		return;

	/* Reset the alternate function */
	mfpr_write(gpio, MFPR_DEFAULT | MFP_AF0);
	direction = strsep(&curr_gpio_opt, " ");
	if (direction == NULL)
		return;

	if (!strcmp(direction, "in")) {
		/* Reset the direction bit - set to input */
		CMX300_GPDR(gpio) &= ~GPIO_bit(gpio);
		return;
	}

	if (!strcmp(direction, "out")) {
		value_str = strsep(&curr_gpio_opt, " ");
		if (value_str == NULL)
			return;

		value = simple_strtoul(value_str, NULL, 10);
		/* Set the direction bit - set to output */
		CMX300_GPDR(gpio) |= GPIO_bit(gpio);

		if (value) {
			/* Set the pin output */
			CMX300_GPSR(gpio) |= GPIO_bit(gpio);
		} else {
			/* Clear the pin output */
			CMX300_GPCR(gpio) |= GPIO_bit(gpio);
		}
	}
}

int set_user_gpios(void)
{
	char *serial;
	char *custom_opt = getenv(GPIO_OPTIONS_ENV_VAR_NAME);
	char *curr_gpio_opt;
	int i;
	unsigned char *options;
	serial_opt_t serialnr = { 0 };

	if (custom_opt == NULL)
		return 0;

	strncpy(gpio_opt_str, custom_opt, 256);
	custom_opt = gpio_opt_str;

	/* Choose the gpio functionality options, according to the revision */
	if (get_board_rev() == 120)
		options = gpio_opt_rev120;
	else if (get_board_rev() >= 130)
		options = gpio_opt_rev130;
	else
		return -1;

	/* Update the board options needed to validate the GPIO values */
	serial = getenv(SERIAL_ENV_VAR_NAME);

	/* Get the audio/touchscreen option: */
	i = get_ser_option('A', serial);
	if (i > 0)
		serialnr.at = serial[i] == 'T' ? 1 : 0;

	/* Get wifi/bluetooth option: */
	serialnr.w = get_ser_option('W', serial) > 0 ? 1 : 0;

	while ((curr_gpio_opt = strsep(&custom_opt, ",")) != NULL)
		set_gpio(curr_gpio_opt, &serialnr, options);

	return 0;
}
