/*
 * board/cmx300/mfp.c
 *
 * MFPR handling and offsets for the CompuLab CM-X300 module
 *
 * (C) Copyright 2010
 * Yulia Vilensky, CompuLab, vilensky@compulab.co.il
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

#include "mfp.h"
#include "cmx300_common.h"

#define MFPR(offset)	__REG(MFP_BASE + (offset))

/*
 * Not all MFP control registeres are included in these tables.
 * Feel free to add the needed registers to the tables, as well as
 * their definitions in mfp.h.
 */

static unsigned int PXA310_MFPR_offset[] = {
	0x0B4, 0x0B8, 0x0BC, 0x27C, 0x280, 0x284, 0x288, 0x28C,	/* MFPRs 0->7 */
	0x290, 0x294, 0x298, 0x29C, 0x2A0, 0x2A4, 0x2A8, 0x2AC,	/* MFPRs 8->15 */
	0x2B0, 0x2B4, 0x2B8, 0x2BC, 0x2C0, 0x2C4, 0x2C8, 0x2CC,	/* MFPRs 16->23 */
	0x2D0, 0x2D4, 0x2D8, 0x400, 0x404, 0x408, 0x418, 0x41C,	/* MFPRs 24->31 */
	0x420, 0x424, 0x428, 0x42C, 0x430, 0x434, 0x438, 0x43C,	/* MFPRs 32->39 */
	0x440, 0x444, 0x448, 0x44C, 0x450, 0x454, 0x458, 0x45C,	/* MFPRs 40->47 */
	0x460, 0x464, 0x468, 0x46C, 0x470, 0x474, 0x478, 0x47C,	/* MFPRs 48->55 */
	0x480, 0x484, 0x488, 0x48C, 0x490, 0x494, 0x498, 0x49C,	/* MFPRs 56->63 */
	0x4A0, 0x4A4, 0x4A8, 0x4AC, 0x4B0, 0x4B4, 0x4B8, 0x4BC,	/* MFPRs 64->71 */
	0x4C0, 0x4C4, 0x4C8, 0x4CC, 0x4D0, 0x4D4, 0x4D8, 0X4DC,	/* MFPRs 72->79 */
	0X4E0, 0x4E4, 0x4E8, 0x4EC, 0x4F0, 0x4F4, 0x4F8, 0x4FC,	/* MFPRs 80->87 */
	0x500, 0x504, 0x508, 0x50C, 0x510, 0x514, 0x518, 0x51C,	/* MFPRs 88->95 */
	0x520, 0x524, 0x528, 0x600, 0x604, 0x608, 0x60C, 0x610,	/* MFPRs 96->103 */
	0x614, 0x618, 0x61C, 0x620, 0x624, 0x628, 0x62C, 0x630,	/* MFPRs 104->111 */
	0x634, 0x638, 0x63C, 0x640, 0x644, 0x648, 0x64C, 0x650,	/* MFPRs 112->119 */
	0x654, 0x658, 0x65C, 0x660, 0x664, 0x668, 0x66C, 0x670,	/* MFPRs 120->127 */
	0x674, 0x678, 0x2DC, 0x2E0, 0x2E4, 0x2E8, 0x2EC, 0x52C,	/* MFPRs 0_2->7_2 */
	0x530, 0x534, 0x538, 0x40C, 0x410, 0x414,		/* MFPRs 8_2->10_2, ULPI pins */
};

static inline unsigned int get_MFPR_offset(unsigned int gpio)
{
	if (cpu_is_pxa300() && (gpio >= 30) && (gpio <= 98))
		return  PXA310_MFPR_offset[gpio] - 0xC;

	return PXA310_MFPR_offset[gpio];
}

void mfpr_write(enum MFPRs mfpr, u32 value)
{
	/* PXA300 has less MFPRs then PXA310 */
	if (cpu_is_pxa300() && (mfpr > MFPR_GPIO6_2))
		return;

	MFPR(get_MFPR_offset(mfpr)) = value;
}

u32 mfpr_read(enum MFPRs mfpr)
{
	return MFPR(get_MFPR_offset(mfpr));
}
