/*
 * board/cmx300/usb.c
 *
 * USB configuration and startup for the CompuLab CM-X300 module
 *
 * (C) Copyright 2009
 * Igor Grinberg, CompuLab, grinberg@compulab.co.il
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>

#ifdef CONFIG_USB_OHCI_NEW

#include <asm/arch/hardware.h>
#include <usb.h>
#include <da9030.h>
#include "da9030.h"
#include "cmx300_common.h"
#include "mfp.h"

#define UP2OCR_DPPDE	(1 << 2)	/* Host Port 2 Transceiver D+ Pull Down Enable */
#define UP2OCR_DMPDE	(1 << 3)	/* Host Port 2 Transceiver D- Pull Down Enable */
#define UP2OCR_DPPUE	(1 << 4)	/* Host Port 2 Transceiver D+ Pull Up Enable */
#define UP2OCR_DMPUE	(1 << 5)	/* Host Port 2 Transceiver D- Pull Up Enable */
#define UP2OCR_DPPUBE	(1 << 6)	/* Host Port 2 Transceiver D+ Pull Up Bypass Enable */
#define UP2OCR_DMPUBE	(1 << 7)	/* Host Port 2 Transceiver D- Pull Up Bypass Enable */
#define UP2OCR_HXS	(1 << 16)	/* Host Port 2 Transceiver Output Select */
#define UP2OCR_HXOE	(1 << 17)	/* Host Port 2 Transceiver Output Enable */

#define UP3OCR_CFG	(0x3 << 0)	/* Host Port 3 configuration bits */

/* OTG */
#define U2DOTGCR	__REG(0x54100020)
#define U2DOTGCR_ULE		(1 << 0)
#define U2DOTGCR_RTSM		(1 << 1)
#define U2DOTGCR_SMAF		(1 << 2)
#define U2DOTGCR_ULAF		(1 << 3)
#define U2DOTGCR_UTMID		(1 << 4)
#define U2DOTGCR_CKAF		(1 << 5)
#define U2DOTGCR_BHNP		(1 << 28)
#define U2DOTGCR_AHNP		(1 << 29)
#define U2DOTGCR_AALTHNP	(1 << 30)
#define U2DOTGCR_OTGEN		(1 << 31)

/* OTG Interrupts */
#define U2DOTGICR	__REG(0x54100024)
#define U2DOTGICR_MASK	0x37F7F

/* PXA310 USB OTG Controller */
#define ULPI_FUNCTION_CONTROL		0x4
#define ULPI_FUNCTION_CONTROL_SET	0x5
#define ULPI_FUNCTION_CONTROL_CLEAR	0x6
#define ULPI_INTERFACE_CONTROL		0x7
#define ULPI_INTERFACE_CONTROL_SET	0x8
#define ULPI_INTERFACE_CONTROL_CLEAR	0x9
#define ULPI_OTG_CONTROL		0xA
#define ULPI_OTG_CONTROL_SET		0xB
#define ULPI_OTG_CONTROL_CLEAR		0xC

#define ULPI_IC_6PIN		(1 << 0)	/* XCVR 6 pin mode */
#define ULPI_IC_3PIN		(1 << 1)	/* XCVR 3 pin mode */
#define ULPI_IC_CARKIT		(1 << 2)	/* Carkit mode */
#define ULPI_IC_CLKSUSPENDM	(1 << 3)	/* Active low clock suspend */

#define ULPI_OC_IDPULLUP	(1 << 0)	/* ID Pull Up, enable sampling of ID line */
#define ULPI_OC_DPPULLDOWN	(1 << 1)	/* Enable the 15K Ohm pull down resistor on D+ */
#define ULPI_OC_DMPULLDOWN	(1 << 2)	/* Enable the 15K Ohm pull down resistor on D- */
#define ULPI_OC_DISCHRGVBUS	(1 << 3)	/* Discharge Vbus */
#define ULPI_OC_CHRGVBUS	(1 << 4)	/* Charge Vbus, for Vbus pulsing SRP */
#define ULPI_OC_DRVVBUS		(1 << 5)	/* Drive 5V on Vbus */
#define ULPI_OC_DRVVBUSEXT	(1 << 6)	/* Drive Vbus using external supply */

/* USB2 Device Control Register */
#define U2DCR		__REG(0x54100000)
#define U2DCR_UDE	(1 << 0)
#define U2DCR_UDA	(1 << 1)
#define U2DCR_UDR	(1 << 2)
#define U2DCR_EMCE	(1 << 3)
#define U2DCR_AAISN	(0xf << 4)
#define U2DCR_AIN	(0xf << 8)
#define U2DCR_ACN	(0xf << 12)
#define U2DCR_DWRE	(1 << 16)
#define U2DCR_SMAC	(1 << 17)
#define U2DCR_HS	(1 << 18)
#define U2DCR_CC	(1 << 19)
#define U2DCR_ADD	(1 << 20)
#define U2DCR_ABP	(1 << 21)
#define U2DCR_UCLKOVR	(1 << 22)
#define U2DCR_FSTC	(0x7 << 24)
#define U2DCR_SPEOREN	(1 << 27)
#define U2DCR_HSTC	(0x7 << 28)
#define U2DCR_NDC	(1 << 31)

/* USB2 OTD ULPI Control Register */
#define U2DOTGUCR		__REG(0x54100030)
#define U2DOTGUCR_RDATA		(0xff << 0)	/* The data for a READ command */
#define U2DOTGUCR_RDATA_S	0		/* shift */
#define U2DOTGUCR_WDATA		(0xff << 8)	/* The data for a WRITE command */
#define U2DOTGUCR_WDATA_S	8		/* shift */
#define U2DOTGUCR_ADDR		(0x3f << 16)	/* Address of ULPI PHY register */
#define U2DOTGUCR_ADDR_S	16		/* shift */
#define U2DOTGUCR_RNW		(1 << 24)	/* Read or Write operation */
#define U2DOTGUCR_RUN		(1 << 25)	/* RUN */

/* Host Port 3 Control Register */
#define U2DP3CR		__REG(0x54100034)	/* U2D Port 3 Control Reg */
#define U2DP3CR_CFG	(0x3 << 0)		/* Host Port 3 Configuration */
#define U2DP3CR_VPVMBEN	(1 << 2)		/* Host Port 3 Vp/Vm Block Enable */
#define U2DP3CR_P3SS	(0x7 << 4)		/* Host Port 3 Serial Mode Select */
#define U2DP3CR_P3SS_S	4			/* shift */
#define U2DP3CR_P2SS	(0x3 << 8)		/* Host Port 2 Serial Mode Select */
#define U2DP3CR_P2SS_S	8			/* shift */

/* U2DC OTG ULPI Status Register */
#define U2DOTGUSR	__REG(0x5410002C)	/* U2D OTG ULPI Status */
#define U2DOTGUSR_CK	(1 << 0)	/* Carkit Interrupt Status */
#define U2DOTGUSR_VV	(1 << 1)	/* OTG Vbus Valid Status */
#define U2DOTGUSR_SV	(1 << 2)	/* OTG Session Valid Status */
#define U2DOTGUSR_SE	(1 << 3)	/* OTG Session End Status */
#define U2DOTGUSR_ID	(1 << 4)	/* OTG IDGnd Status */
#define U2DOTGUSR_LS0	(1 << 5)	/* RXCMD Linestate 0 Status */
#define U2DOTGUSR_LS1	(1 << 6)	/* RXCMD Linestate 1 Status */
#define U2DOTGUSR_CKA	(1 << 28)	/* ULPI Car Kit Mode Active */
#define U2DOTGUSR_S3A	(1 << 29)	/* ULPI Serial Mode (3-pin) Active */
#define U2DOTGUSR_S6A	(1 << 30)	/* ULPI Serial Mode (6-pin) Active */
#define U2DOTGUSR_LPA	(1 << 31)	/* ULPI Low Power Mode Active */


enum u2d_phy_mode {
	SYNCH = 0,
	CARKIT = 0x1,
	SER_3PIN = 0x2,
	SER_6PIN = 0x4,
	LOWPOWER = 0x8,
	/* rtsm mode */
	PRE_SYNCH = 0x10,
};

static int __maybe_unused ulpi_reg_read(u8 reg, u8 *value)
{
	int i = 50000;
	enum u2d_phy_mode state = (U2DOTGUSR & 0xF0000000) >> 28;

	if ((state != SYNCH) && (state != PRE_SYNCH)) {
		printf(": not in SYNCH mode!!!");
		return -1;
	}

	U2DOTGUCR = U2DOTGUCR_RUN | U2DOTGUCR_RNW | (reg << U2DOTGUCR_ADDR_S);

	while ((U2DOTGUCR & U2DOTGUCR_RUN) && i--) ;

	if (i <= 0) {
		printf("Read ULPI register Time out,"
			" reg %x otgucr %x, usr %x ucr %x\n",
			reg, U2DOTGUCR, U2DOTGUSR, U2DOTGCR);
		return -1;
	}

	*value = (u8)(U2DOTGUCR & U2DOTGUCR_RDATA);

	return 0;
}

static int ulpi_reg_write(u8 reg, u8 value)
{
	int i = 50000;
	enum u2d_phy_mode state = (U2DOTGUSR & 0xF0000000) >> 28;

	if ((state != SYNCH) && (state != PRE_SYNCH)) {
		printf(": not in SYNCH mode!!!");
		return -1;
	}

	U2DOTGUCR = U2DOTGUCR_RUN | (reg << U2DOTGUCR_ADDR_S) \
			| (value << U2DOTGUCR_WDATA_S);

	while ((U2DOTGUCR & U2DOTGUCR_RUN) && i--) ;

	if (i <= 0) {
		printf("Write ULPI register Time out,"
				" reg %x val %x\n", reg, (int)value);
		return -1;
	}

	return 0;
}

static void pxa310_ulpi_mfp_setup(void)
{
	uint mask = (MFPR_DEFAULT & ~(MFPR_DRV_MASK | MFPR_LPM_MASK))
		    | (MFP_DS01X << MFP_DRIVE_S) | MFP_SLEEP_OE_N
		    | MFP_PULLDOWN_EN | MFP_AF0;

	/* ULPI_STP */
	mfpr_write(MFPR_ULPI_STP, mask);
	/* ULPI_NXT */
	mfpr_write(MFPR_ULPI_NXT, mask);
	/* ULPI_DIR */
	mfpr_write(MFPR_ULPI_DIR, mask);

	mask = (MFPR_DEFAULT & ~MFPR_LPM_MASK) | MFP_SLEEP_OE_N
		| MFP_SLEEP_DATA | MFP_PULLUP_EN | MFP_AF3;

	/* GPIO30_ULPI_DATA_OUT_0 */
	mfpr_write(MFPR_GPIO30, mask);
	/* GPIO31_ULPI_DATA_OUT_1 */
	mfpr_write(MFPR_GPIO31, mask);
	/* GPIO32_ULPI_DATA_OUT_2 */
	mfpr_write(MFPR_GPIO32, mask);
	/* GPIO33_ULPI_DATA_OUT_3 */
	mfpr_write(MFPR_GPIO33, mask);
	/* GPIO34_ULPI_DATA_OUT_4 */
	mfpr_write(MFPR_GPIO34, mask);
	/* GPIO35_ULPI_DATA_OUT_5 */
	mfpr_write(MFPR_GPIO35, mask);
	/* GPIO36_ULPI_DATA_OUT_6 */
	mfpr_write(MFPR_GPIO36, mask);
	/* GPIO37_ULPI_DATA_OUT_7 */
	mfpr_write(MFPR_GPIO37, mask);
	/* GPIO38_ULPI_CLK */
	mfpr_write(MFPR_GPIO38, mask);
	/* GPIO127_GPIO - PHY reset pin */
	mfpr_write(MFPR_GPIO127, MFPR_DEFAULT | MFP_AF0);
}

static void usb_port3_mfp_config(void)
{
	uint mask = (MFPR_DEFAULT & ~MFP_EDGE_CLEAR) | MFP_AF2;

	/* GPIO77_USB_P3_1 */
	mfpr_write(MFPR_GPIO77, mask);
	/* GPIO78_USB_P3_2 */
	mfpr_write(MFPR_GPIO78, mask);
	/* GPIO79_USB_P3_3 */
	mfpr_write(MFPR_GPIO79, mask);
	/* GPIO80_USB_P3_4 */
	mfpr_write(MFPR_GPIO80, mask);
	/* GPIO81_USB_P3_5 */
	mfpr_write(MFPR_GPIO81, mask);
	/* GPIO82_USB_P3_6 */
	mfpr_write(MFPR_GPIO82, mask);
	/* GPIO0_2_USBH_PEN */
	mfpr_write(MFPR_GPIO0_2, (MFPR_DEFAULT & ~MFP_EDGE_CLEAR) | MFP_AF1);
}

static void ulpi_phy_reset(void)
{
	GPDR3 |= GPIO_bit(31);		/* GPIO127 */
	GPCR3 |= GPIO_bit(31);
	mdelay(10);
	GPSR3 |= GPIO_bit(31);
}

static void otg_init(void)
{
	U2DOTGCR = (U2DOTGCR | U2DOTGCR_ULAF | U2DOTGCR_UTMID) &
			~(U2DOTGCR_SMAF | U2DOTGCR_CKAF);

	U2DOTGCR |= U2DOTGCR_ULE;

	/* according to pxa3xx manual vol. IV (p.145) */
	/* here we need to enable OTG interrups */
	/* U2DOTGICR |= U2DOTGICR_MASK; */
	/* but there is no one to handle them in U-Boot */
}

static void ulpi_rtsm(void)
{
	U2DOTGCR |= U2DOTGCR_RTSM;
	mdelay(10);

	/* setup OTG sync mode */
	U2DOTGCR = (U2DOTGCR | U2DOTGCR_ULAF) & ~(U2DOTGCR_SMAF | U2DOTGCR_CKAF);
}

static void set_ulpi_serial6(void)
{
	/* enable the 15K Ohm pull-down resistor on D+ & D- and drive 5V */
	ulpi_reg_write(ULPI_OTG_CONTROL_SET, ULPI_OC_DPPULLDOWN);
	ulpi_reg_write(ULPI_OTG_CONTROL_SET, ULPI_OC_DMPULLDOWN);
	ulpi_reg_write(ULPI_OTG_CONTROL_SET, ULPI_OC_DRVVBUS);

	/* configure UHC for 6-pin serial mode */
	U2DP3CR &= ~(U2DP3CR_P2SS);

	/* set PHY into host and serial mode */
	ulpi_reg_write(ULPI_FUNCTION_CONTROL_SET, 0x45);
	ulpi_reg_write(ULPI_INTERFACE_CONTROL, ULPI_IC_6PIN);

	/* enable the serial mode */
	U2DOTGCR = (U2DOTGCR | U2DOTGCR_SMAF) & ~(U2DOTGCR_ULAF | U2DOTGCR_CKAF);
}

static void pxa310_otg_host_init(void)
{
	/* setup the GPIOs for ULPI */
	pxa310_ulpi_mfp_setup();

	/* reset the PHY */
	ulpi_phy_reset();

	/* setup OTG */
	otg_init();

	/* put PHY to sync mode */
	ulpi_rtsm();

	/* disable USB2 Device (slave) controller */
	U2DCR &= ~U2DCR_UDE;
	U2DOTGCR |= U2DOTGCR_UTMID;

	/* according to pxa3xx manual vol. IV (p.145) */
	/* here also we need to enable OTG interrups */
	/* U2DOTGICR |= U2DOTGICR_MASK; */
	/* we don't need this for the same reason as above */

	set_ulpi_serial6();
}

void pxa310_usb_port3_phy_suspend(void)
{
	/* set AF to GPIO */
	mfpr_write(MFPR_GPIO78, MFPR_DEFAULT | MFP_AF0);
	mfpr_write(MFPR_GPIO107, MFPR_DEFAULT | MFP_AF0);
	mfpr_write(MFPR_GPIO0_2, MFPR_DEFAULT | MFP_AF0);

	/* set direction to output */
	GPDR2 |= GPIO_bit(78);
	GPDR3 |= GPIO_bit(107);
	GPDR0 |= GPIO_bit(0);

	/* set values */
	GPSR2 |= GPIO_bit(78);
	GPCR3 |= GPIO_bit(107);
	GPSR0 |= GPIO_bit(0);
}

void pxa310_usb_port3_phy_resume(void)
{
	/* set AF to GPIO */
	mfpr_write(MFPR_GPIO78, MFPR_DEFAULT | MFP_AF0);
	mfpr_write(MFPR_GPIO107, MFPR_DEFAULT | MFP_AF0);
	mfpr_write(MFPR_GPIO0_2, MFPR_DEFAULT | MFP_AF0);

	/* set direction to output */
	GPDR2 |= GPIO_bit(78);
	GPDR3 |= GPIO_bit(107);
	GPDR0 |= GPIO_bit(0);

	/* set values */
	GPCR2 |= GPIO_bit(78);
	GPSR3 |= GPIO_bit(107);
	GPCR0 |= GPIO_bit(0);
}

int usb_board_init(void)
{
	struct da9030_ops *ops = da9030_get_ops();
	u32 val;

	/* Enable USB Charge Pump for rev. >=1.3 */
	if (get_board_rev() >= 130 && is_adv_pwr_mod_enabled()) {
		ops->set_reg(MISC_CONTROLB, MISCB_SESSION_VALID_ENABLE);
		val = USB_PUMP_EN_USBVEP | USB_PUMP_EN_USBVE | USB_PUMP_USBVE;
		ops->set_reg(USBPUMP, val);
		mdelay(100);
	}

	if (cpu_is_pxa300()) {
		UP2OCR = UP2OCR_HXS | UP2OCR_HXOE | UP2OCR_DMPDE | UP2OCR_DPPDE;
		UP3OCR = 0;
	}

	if (cpu_is_pxa310()) {
		if (is_adv_pwr_mod_enabled()) {
			/* Enable LDO4 (USB ULPI) and LDO7 (USB) */
			ops->get_reg(REG_CONTROL_1_97, &val);
			val |= (RC3_LDO4_EN | RC3_LDO7_EN);
			ops->set_reg(REG_CONTROL_1_97, val);

			pxa310_usb_port3_phy_resume();
			mdelay(1);
		}

		/* CLK_POUT is connected directly to the ULPI PHY */
		/* Turn CLK_POUT on */
		OSCC |= OSCC_PEN;
		/* Turn U2D clock on */
		CKENA |= CKENA_6_USBCLI;

		pxa310_otg_host_init();

		usb_port3_mfp_config();

		/* U2D Port 3 init */
		U2DP3CR &= ~(U2DP3CR_P3SS | U2DP3CR_CFG);
	}

	UHCHR = (UHCHR | UHCHR_PCPL) & ~(UHCHR_SSEP1 | UHCHR_SSEP2 | UHCHR_SSE);

	return 0;
}

int usb_board_stop(void)
{
	struct da9030_ops *ops = da9030_get_ops();
	u32 val;

	/* Disable USB Charge Pump for rev. >=1.3 */
	if (get_board_rev() >= 130 && is_adv_pwr_mod_enabled())
		ops->set_reg(USBPUMP, USB_PUMP_EN_USBVEP | USB_PUMP_EN_USBVE);

	if (cpu_is_pxa310()) {
		/* Turn CLK_POUT off to save some power */
		OSCC &= ~OSCC_PEN;

		if (is_adv_pwr_mod_enabled()) {
			/* Disable LDO4 (USB ULPI) and LDO7 (USB) */
			ops->get_reg(REG_CONTROL_1_97, &val);
			val &= ~(RC3_LDO4_EN | RC3_LDO7_EN);
			ops->set_reg(REG_CONTROL_1_97, val);

			pxa310_usb_port3_phy_suspend();
			mdelay(1);
		}
	}

	return 0;
}

int usb_board_init_fail(void)
{
	return 0;
}

#endif /* CONFIG_USB_OHCI_NEW */
