/*
 * (C) Copyright 2006 DENX Software Engineering
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/* DA9030 chip address definition */
#define DA9030_ADDR	0x92

/* DA9030 register definitions */
#define CID			0x00
#define EVENT_A			0x01
#define EVENT_A_CHIOVER		(1 << 7)
#define EVENT_A_VBAT_MON_TXON	(1 << 6)
#define EVENT_A_VBAT_MON	(1 << 5)
#define EVENT_A_TBAT		(1 << 4)
#define EVENT_A_CHDET		(1 << 3)
#define EVENT_A_EXTON		(1 << 2)
#define EVENT_A_PWREN1		(1 << 1)
#define EVENT_A_ONKEY_N		(1 << 0)

#define EVENT_B			0x02
#define EVENT_B_WDOG_INT	(1 << 7)
#define EVENT_B_SRP_DETECT	(1 << 6)
#define EVENT_B_SESSION_VALID	(1 << 5)
#define EVENT_B_VBUS_VALID_4_0	(1 << 4)
#define EVENT_B_VBUS_VALID_4_4	(1 << 3)
#define EVENT_B_ADC_READY	(1 << 2)
#define EVENT_B_CCTO		(1 << 1)
#define EVENT_B_TCTO		(1 << 0)

#define EVENT_C			0x03
#define EVENT_C_ADC_IN5		(1 << 7)
#define EVENT_C_ADC_IN4		(1 << 6)
#define EVENT_C_BUCK2		(1 << 5)
#define EVENT_C_LDO19		(1 << 4)
#define EVENT_C_LDO18		(1 << 3)
#define EVENT_C_LDO17		(1 << 2)
#define EVENT_C_LDO16		(1 << 1)
#define EVENT_C_LDO15		(1 << 0)

#define STATUS			0x04
#define STATUS_MCLK_DETECT	(1 << 7)
#define STATUS_VBAT_MON_TXON	(1 << 6)
#define STATUS_VBAT_MON		(1 << 5)
#define STATUS_TBAT		(1 << 4)
#define STATUS_CHDET		(1 << 3)
#define STATUS_EXTON		(1 << 2)
#define STATUS_PWREN1		(1 << 1)
#define STATUS_ONKEY_N		(1 << 0)

#define IRQ_MASK_A			0x05
#define IRQ_MASK_A_CHIOVER		(1 << 7)
#define IRQ_MASK_A_VBAT_MON_TXON	(1 << 6)
#define IRQ_MASK_A_VBAT_MON		(1 << 5)
#define IRQ_MASK_A_TBAT			(1 << 4)
#define IRQ_MASK_A_CHDET		(1 << 3)
#define IRQ_MASK_A_EXTON		(1 << 2)
#define IRQ_MASK_A_PWREN1		(1 << 1)
#define IRQ_MASK_A_ONKEY_N		(1 << 0)

#define IRQ_MASK_B			0x06
#define IRQ_MASK_B_WDOG_INT		(1 << 7)
#define IRQ_MASK_B_SRP_DETECT		(1 << 6)
#define IRQ_MASK_B_SESSION_VALID	(1 << 5)
#define IRQ_MASK_B_VBUS_VALID_4_0	(1 << 4)
#define IRQ_MASK_B_VBUS_VALID_4_4	(1 << 3)
#define IRQ_MASK_B_ADC_READY		(1 << 2)
#define IRQ_MASK_B_CCTO			(1 << 1)
#define IRQ_MASK_B_TCTO			(1 << 0)

#define IRQ_MASK_C		0x07
#define IRQ_MASK_C_ADC_IN5	(1 << 7)
#define IRQ_MASK_C_ADC_IN4	(1 << 6)
#define IRQ_MASK_C_BUCK2	(1 << 5)
#define IRQ_MASK_C_LDO19	(1 << 4)
#define IRQ_MASK_C_LDO18	(1 << 3)
#define IRQ_MASK_C_LDO17	(1 << 2)
#define IRQ_MASK_C_LDO16	(1 << 1)
#define IRQ_MASK_C_LDO15	(1 << 0)

#define SYS_CTRL_A				0x08
#define SYS_CONTROL_A_SLEEP_N_PIN_ENABLE	0x1
#define SYS_CONTROL_A_SHUT_DOWN			(1<<1)
#define SYS_CONTROL_A_HWRES_ENABLE		(1<<2)
#define SYS_CONTROL_A_WDOG_ACTION		(1<<3)
#define SYS_CONTROL_A_WATCHDOG			(1<<7)

#define SYS_CONTROL_B			0x09
#define SYS_CLTR_B_SLEEP_13MHZ_EN	(1 << 4)
#define SYS_CLTR_B_AUTO_CLK_SWITCH	(1 << 3)

#define FAULT_LOG		0x0A
#define LDO_10_11		0x10
#define LDO_10_11_LDO10_3V	0xC
#define LDO_10_11_LDO11_3V2	(0xE << 4)

#define LDO_15			0x11
#define LDO_14_16		0x12
#define LDO_18_19		0x13
#define LDO_18_19_LDO18_3V2	0xE

#define LDO_17_SIMCP0		0x14
#define LDO_17_SIMCP0_LDO17_3V	0x0F

#define BUCK2_DVC1		0x15
#define BUCK2_GO		(1 << 7)
#define BUCK2_SLEEP		(1 << 6)
#define BUCK2_TRIM_1V375	0x15
#define BUCK2_TRIM_1V4		0x16
#define BUCK2_TRIM_1V5		0x1A

#define BUCK2_DVC2		0x16

#define REG_CONTROL_1_17	0x17
#define RC1_LDO17_EN		(1 << 7)
#define RC1_LDO16_EN		(1 << 6)
#define RC1_LDO15_EN		(1 << 5)
#define RC1_LDO14_EN		(1 << 4)
#define RC1_LDO13_EN		(1 << 3)
#define RC1_LDO11_EN		(1 << 2)
#define RC1_LDO10_EN		(1 << 1)
#define RC1_BUCK2_EN		(1 << 0)

#define REG_CONTROL_2_18	0x18
#define RC2_SIMCP_EN		(1 << 6)
#define RC2_LDO19_EN		(1 << 1)
#define RC2_LDO18_EN		(1 << 0)

#define USBPUMP			0x19
#define USB_PUMP_EN_USBVEP	(1 << 7)
#define USB_PUMP_EN_USBVE	(1 << 6)
#define USB_PUMP_SPR_DETECT	(1 << 5)
#define USB_PUMP_SESSION_VALID	(1 << 4)
#define USB_PUMP_VBUS_VALID_4_0	(1 << 3)
#define USB_PUMP_VBUS_VALID_4_4	(1 << 2)
#define USB_PUMP_USBVEP		(1 << 1)
#define USB_PUMP_USBVE		(1 << 0)

#define SLEEP_CONTROL			0x1A
#define APP_SLEEP_CTL_PWR_EN		(1 << 7)
#define APP_SLEEP_CTL_SYS_EN		(1 << 6)
#define APP_SLEEP_CTL_BYPASS_LDO19	(1 << 2)
#define APP_SLEEP_CTL_BYPASS_LDO18	(1 << 1)
#define APP_SLEEP_CTL_BYPASS_LDO17	(1 << 0)

#define STARTUP_CONTROL			0x1B
#define STARTUP_CTL_LDO11_PWR_SYS	(1 << 3)
#define STARTUP_CTL_LDO10_PWR_SYS	(1 << 2)
#define STARTUP_CTL_LDO11_START		(1 << 1)
#define STARTUP_CTL_LDO10_START		(1 << 0)

#define LED1_CONTROL		0x20
#define LED2_CONTROL		0x21
#define LED1_ENABLE		(1 << 7)

#define LED3_CONTROL		0x22
#define LED4_CONTROL		0x23
#define LEDPC_CONTROL		0x24
#define WLED_CONTROL		0x25
#define WLED_CP_ENABLE		(1 << 6)
#define WLED_ISET_10		0x3

#define MISC_CONTROLA			0x26
#define MISC_CONTROLB			0x27
#define MISCB_SESSION_VALID_ENABLE	(1 << 3)
#define MISCB_USB_INT_RISING		(1 << 2)
#define MISCB_I2C_ADDRESS		(1 << 1)
#define MISCB_STARTUP_SEQUENCE		(1 << 0)

#define CHARGE_CONTROL		0x28
#define CHRG_CHARGER_ENABLE	(1 << 7)
#define CHRG_ISET_300		(3 << 3)
#define CHRG_ISET_400		(4 << 3)
#define CHRG_ISET_500		(5 << 3)
#define CHRG_ISET_1000		(10 << 3)
#define CHRG_VSET_4V2		0x4

#define CCTR_CONTROL		0x29
#define CCTR_SET_8MIN		0x01

#define TCTR_CONTROL		0x2A
#define CHARGE_PULSE		0x2B

/* ... some missing ...*/

#define LDO1			0x90
#define LDO1_UNLOCK		(0x5 << 5)
#define LDO1_TRIM_3V		0x12

#define LDO2_3			0x91
#define LDO2_3_LDO2_3V2		(0xE << 4)
#define LDO2_3_LDO3_3V		0xC

#define LDO4_5			0x92
#define LDO6_SIMCP		0x93
#define LDO6_SIMCP_LDO6_3V2	0xE

#define LDO7_8			0x94
#define LDO9_12			0x95
#define BUCK			0x96
#define REG_CONTROL_1_97	0x97
#define RC3_LDO7_EN		(1 << 7)
#define RC3_LDO6_EN		(1 << 6)
#define RC3_LDO5_EN		(1 << 5)
#define RC3_LDO4_EN		(1 << 4)
#define RC3_LDO3_EN		(1 << 3)
#define RC3_LDO2_EN		(1 << 2)
#define RC3_LDO1_EN		(1 << 1)
#define RC3_BUCK_EN		(1 << 0)

#define REG_CONTROL_2_98	0x98
#define RC4_SLEEP		(1 << 7)
#define RC4_SIMCP_ENABLE	(1 << 6)
#define RC4_LDO14_EN		(1 << 5)
#define RC4_LDO12_EN		(1 << 4)
#define RC4_LDO11_EN		(1 << 3)
#define RC4_LDO10_EN		(1 << 2)
#define RC4_LDO9_EN		(1 << 1)
#define RC4_LDO8_EN		(1 << 0)

#define REG_SLEEP_CONTROL1	0x99
#define REG_SLEEP_CONTROL2	0x9A
#define REG_SLEEP_CONTROL3	0x9B

#define ADC_MAN_CONTROL			0xA0
#define ADC_DEBOUNCE_VBATMON_TXON	(1 << 7)
#define ADC_DEBOUNCE_VBATMON		(1 << 6)
#define ADC_TBATREF_ENABLE		(1 << 5)
#define ADC_LDO_INT_ENABLE		(1 << 4)
#define ADC_MAN_CONV			(1 << 3)
#define ADC_MUX_IN5			0x5
#define ADC_MUX_IN4			(1 << 2)
#define ADC_MUX_TBAT			0x3
#define ADC_MUX_VCH			(1 << 1)
#define ADC_MUX_ICH			(1 << 0)
#define ADC_MUX_VBAT			0

#define ADC_AUTO_CONTROL	0xA1
#define ADC_IN5_EN		(1 << 7)
#define ADC_IN4_EN		(1 << 6)
#define ADC_TBAT_ENABLE		(1 << 5)
#define ADC_VBAT_IN_TXON	(1 << 4)
#define ADC_VCH_ENABLE		(1 << 3)
#define ADC_ICH_ENABLE		(1 << 2)
#define ADC_VBAT_ENABLE		(1 << 1)
#define ADC_AUDIO_SLEEP_ENABLE	(1 << 0)

#define VBATMON			0xA2
#define VBATMONTXMON		0xA3
#define TBATHIGHP		0xA4
#define TBATHIGHN		0xA5
#define TBATLOW			0xA6
#define MAN_RES			0xB0
#define VBAT_RES		0xB1
#define VBATMIN_RES		0xB2
#define VBATMINTXON_RES		0xB3
#define ICHMAX_RES		0xB4
#define ICHMIN_RES		0xB5
#define ICHAVERAGE_RES		0xB6
#define VCHMAX_RES		0xB7
#define VCHMIN_RES		0xB8
#define TBAT_RES		0xB9
#define ADC_IN4_RES		0xBA

